@@ -3,9 +3,9 @@
  */
 
 /*
- * $Id: utmp_ph.c 8 2008-05-28 09:27:26Z rmanfredi $
+ * $Id: utmp_ph.c,v 3.0.1.1 1994/10/29 18:13:23 ram Exp $
  *
- *  Copyright (c) 1990-2006, Raphael Manfredi
+ *  Copyright (c) 1990-1993, Raphael Manfredi
  *  
  *  You may redistribute only under the terms of the Artistic License,
  *  as specified in the README file that comes with the distribution.
@@ -40,8 +40,6 @@
 #define MAX_LEN		1024		/* Maximum length for strings */
 #define PADSTR		"..pad.. "	/* Pad string, for comment */
 
-#include <stdio.h>
-
 #ifdef I_SYS_TYPES
 #include <sys/types.h>
 #endif
@@ -52,10 +50,6 @@
 #include <strings.h>
 #endif
 
-#ifdef I_STDLIB
-#include <stdlib.h>
-#endif
-
 #include <utmp.h>
 #include "confmagic.h"
 
@@ -76,7 +70,7 @@ char *padstr = PADSTR;
 		strcat(fields, "'line', "); \
 		last_off += line_len;
 
-int main()
+main()
 {
 	struct utmp *utmp = (struct utmp *) 0;
 	char comment[MAX_LEN];
