@@ -1,8 +1,8 @@
 # Set up mailagent and filter paths
 
-;# $Id: init.pl 1 2006-08-24 13:24:12Z rmanfredi $
+;# $Id: init.pl,v 3.0.1.2 1995/08/07 16:28:52 ram Exp $
 ;#
-;#  Copyright (c) 1990-2006, Raphael Manfredi
+;#  Copyright (c) 1990-1993, Raphael Manfredi
 ;#  
 ;#  You may redistribute only under the terms of the Artistic License,
 ;#  as specified in the README file that comes with the distribution.
@@ -11,9 +11,6 @@
 ;#  of the source tree for mailagent 3.0.
 ;#
 ;# $Log: init.pl,v $
-;# Revision 3.0.1.3  1999/01/13  18:16:59  ram
-;# patch64: cleanup agent.wait file since now always produced
-;#
 ;# Revision 3.0.1.2  1995/08/07  16:28:52  ram
 ;# patch37: added support for locking on filesystems with short filenames
 ;#
@@ -34,5 +31,4 @@ $filter = "$up/filter/filter";
 
 # Make sure no lock were left by previous test
 unlink "$pwd/out/filter$lockext", "$pwd/out/perl$lockext";
-unlink 'agent.wait';
 
